@@ -1,6 +1,6 @@
 ;;; helm-buffers.el --- helm support for buffers. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2015 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2016 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -324,10 +324,10 @@ See `ido-make-buffer-list' for more infos."
          (size (propertize (helm-buffer-size buf)
                            'face 'helm-buffer-size))
          (proc (get-buffer-process buf))
-         (dir (with-current-buffer buffer (abbreviate-file-name default-directory)))
+         (dir (with-current-buffer buffer (helm-aif default-directory (abbreviate-file-name it))))
          (file-name (helm-aif (buffer-file-name buf) (abbreviate-file-name it)))
          (name (buffer-name buf))
-         (name-prefix (when (file-remote-p dir)
+         (name-prefix (when (and dir (file-remote-p dir))
                         (propertize "@ " 'face 'helm-ff-prefix))))
     ;; No fancy things on remote buffers.
     (if (and name-prefix helm-buffer-skip-remote-checking)
@@ -516,7 +516,7 @@ i.e same color."
   (let ((regexp-list (cl-loop with pattern = helm-pattern
                               for p in (split-string pattern)
                               unless (string-match
-                                      "\\`\\(\\*\\)\\|\\(/\\)\\|\\(@\\)" p)
+                                      "\\`\\(\\*\\|/\\|@\\)" p)
                               collect p)))
     (if regexp-list
         (cl-loop for re in regexp-list
