@@ -1,6 +1,6 @@
 ;;; helm-lib.el --- Helm routines. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015  Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2015 ~ 2016  Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; Author: Thierry Volpiatto <thierry.volpiatto@gmail.com>
 ;; URL: http://github.com/emacs-helm/helm
@@ -110,14 +110,36 @@ If NAME returns nil the pair is skipped.
            do (setq name (funcall name))
            when name
            collect (cons name (cadr i))))
-
+
+;;; Anaphoric macros.
+;;
 (defmacro helm-aif (test-form then-form &rest else-forms)
-  "Like `if' but set the result of TEST-FORM in a temprary variable called `it'.
+  "Anaphoric version of `if'.
+Like `if' but set the result of TEST-FORM in a temporary variable called `it'.
 THEN-FORM and ELSE-FORMS are then excuted just like in `if'."
   (declare (indent 2) (debug t))
   `(let ((it ,test-form))
      (if it ,then-form ,@else-forms)))
 
+(defmacro helm-awhile (sexp &rest body)
+  "Anaphoric version of `while'."
+  (helm-with-gensyms (flag)
+    `(let ((,flag t))
+       (while ,flag
+         (helm-aif ,sexp
+             (progn ,@body)
+           (setq ,flag nil))))))
+
+(defmacro helm-acond (&rest clauses)
+  "Anaphoric version of `cond'."
+  (unless (null clauses)
+    (helm-with-gensyms (sym)
+      (let ((clause1 (car clauses)))
+        `(let ((,sym ,(car clause1)))
+           (helm-aif ,sym
+               ,@(cdr clause1)
+             (helm-acond ,@(cdr clauses))))))))
+
 (defun helm-current-line-contents ()
   "Current line string without properties."
   (buffer-substring-no-properties (point-at-bol) (point-at-eol)))
@@ -282,9 +304,7 @@ Default is `eq'."
   (cl-loop with cont = (make-hash-table :test test)
         for elm in seq
         unless (gethash elm cont)
-        do (puthash elm elm cont)
-        finally return
-        (cl-loop for i being the hash-values in cont collect i)))
+        collect (puthash elm elm cont)))
 
 (defun helm-skip-entries (seq black-regexp-list &optional white-regexp-list)
   "Remove entries which matches one of REGEXP-LIST from SEQ."
@@ -348,11 +368,13 @@ ARGS is (cand1 cand2 ...) or ((disp1 . real1) (disp2 . real2) ...)
 
 ;;; Strings processing.
 ;;
-(defun helm-stringify (str-or-sym)
-  "Get string of STR-OR-SYM."
-  (if (stringp str-or-sym)
-      str-or-sym
-    (symbol-name str-or-sym)))
+(defun helm-stringify (elm)
+  "Return the representation of ELM as a string.
+ELM can be a string, a number or a symbol."
+  (cl-typecase elm
+    (string elm)
+    (number (number-to-string elm))
+    (symbol (symbol-name elm))))
 
 (defun helm-substring (str width)
   "Return the substring of string STR from 0 to WIDTH.
@@ -415,7 +437,8 @@ Add spaces at end if needed to reach WIDTH when STR is shorter than WIDTH."
     (intern str-or-sym)))
 
 (defun helm-symbol-name (obj)
-  (if (or (consp obj) (byte-code-function-p obj))
+  (if (or (and (consp obj) (functionp obj))
+          (byte-code-function-p obj))
       "Anonymous"
       (symbol-name obj)))
 
@@ -464,10 +487,15 @@ See `kill-new' for argument REPLACE."
 (defun helm-basename (fname &optional ext)
   "Print FNAME  with any  leading directory  components removed.
 If specified, also remove filename extension EXT.
-Arg EXT can be specified as a string with or without dot."
+Arg EXT can be specified as a string with or without dot,
+in this case it should match file-name-extension.
+It can also be non-nil (`t') in this case no checking
+of file-name-extension is done and the extension is removed
+unconditionally."
   (let ((non-essential t))
     (if (and ext (or (string= (file-name-extension fname) ext)
-                     (string= (file-name-extension fname t) ext))
+                     (string= (file-name-extension fname t) ext)
+                     (eq ext t))
              (not (file-directory-p fname)))
         (file-name-sans-extension (file-name-nondirectory fname))
       (file-name-nondirectory (directory-file-name fname)))))
@@ -562,7 +590,8 @@ instead of `helm-walk-ignore-directories'."
   "Same as `file-expand-wildcards' but allow recursion.
 Recursion happen when PATTERN starts with two stars.
 Directories expansion is not supported."
-  (let ((bn (helm-basename pattern)))
+  (let ((bn (helm-basename pattern))
+        (case-fold-search nil))
     (if (and helm-file-globstar
              (string-match "\\`\\*\\{2\\}\\(.*\\)" bn))
         (helm-walk-directory (helm-basedir pattern)
